package yaes.framework.entity.control.accelerationcontrol;

import java.util.ArrayList;
import java.util.List;

import yaes.framework.entity.control.ControlException;
import yaes.framework.entity.control.IAccelerationControl;
import yaes.framework.entity.control.IAccelerationControllable;
import yaes.framework.world.map.Acceleration;
import yaes.framework.world.map.Location;
import yaes.framework.world.map.Speed;

/**
 * @author lboloni
 * 
 */
public class MultiAccelerationControl implements IAccelerationControl,
		IAccelerationControllable {
	private IAccelerationControllable controllable;
	private final List<IAccelerationControl> controllerLists;

	/**
	 * 
	 */
	public MultiAccelerationControl() {
		controllerLists = new ArrayList<IAccelerationControl>();
	}

	public void addController(IAccelerationControl control) {
		control.setControllable(this);
		controllerLists.add(control);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see simulation.control.IControl#control()
	 */
	public void control(double time) throws ControlException {
		controllable.controlAcceleration(new Acceleration(0, 0));
		for (IAccelerationControl element : controllerLists) {
			element.control(time);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see simulation.control.IAccelerationControllable#controlAcceleration(simulation.environment.maps.Acceleration)
	 */
	public void controlAcceleration(Acceleration acceleration)
			throws ControlException {
		// summary
		final Acceleration prev = controllable.getAcceleration();
		final double newX = prev.getXAcceleration()
				+ acceleration.getXAcceleration();
		final double newY = prev.getYAcceleration()
				+ acceleration.getYAcceleration();
		final Acceleration newAcc = new Acceleration(newX, newY);
		controllable.controlAcceleration(newAcc);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see simulation.vehicles.IMovingObject#getAcceleration()
	 */
	public Acceleration getAcceleration() {
		return controllable.getAcceleration();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see simulation.vehicles.IMovingObject#getLocation()
	 */
	public Location getLocation() {
		return controllable.getLocation();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see simulation.vehicles.IMovingObject#getSpeed()
	 */
	public Speed getSpeed() {
		return controllable.getSpeed();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see simulation.control.IAccelerationControl#setControllable(simulation.control.IAccelerationControllable)
	 */
	public void setControllable(IAccelerationControllable controllable) {
		this.controllable = controllable;
	}

	public void setLocation(Location location) {
		controllable.setLocation(location);
	}

	public void updateLocation(double time) {
	}
}
