package yaes.framework.entity.vehicle;

import yaes.framework.world.map.AbstractMovingObject;
import yaes.framework.world.map.INamed;
import yaes.framework.world.map.IPathRecorder;
import yaes.framework.world.map.Location;
import yaes.framework.world.map.Path;

/**
 * This is a moving object which is an external, moving object. It is not under
 * our control, so there is no agent attached to it.
 * 
 * @author lboloni
 * 
 */
public class ObservedMovingObject extends AbstractMovingObject implements
		IPathRecorder, INamed {

	private boolean isRecording = false;
	private Path recordedPath;
	private String name;

	public ObservedMovingObject(String name, Location location) {
		this.name = name;
		setLocation(location);
	}

	@Override
	public String getName() {
		return name;
	}

	@Override
	public Path getRecordedPath() {
		return recordedPath;
	}

	@Override
	public void startPathRecording() {
		isRecording = true;
		recordedPath = new Path();
	}

	@Override
	public void stopPathRecording() {
		isRecording = false;
		recordedPath = null;
	}

	@Override
	public String toString() {
		return "ObservedMovingObject [ " + name + " ] at " + getLocation();
	}

	public void update(double time, Location location) {
		if (isRecording) {
			recordedPath.addLocation(getLocation());
		}
		setLocation(location);
	}

}
