/*
 * Created on Sep 25, 2004
 *
 * To change the template for this generated file go to
 * Window - Preferences - Java - Code Generation - Code and Comments
 */
package yaes.framework.world;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Observable;
import java.util.Set;

import yaes.framework.agent.Directory;
import yaes.framework.simulation.SimulationOutput;
import yaes.framework.world.map.IMap;
import yaes.framework.world.map.NamedLocation;

/**
 * A generic repository of objects.
 * 
 * @author Lotzi Boloni
 * 
 */
/**
 * @author Lotzi Boloni
 * 
 */
public class World extends Observable {
	private final SimulationOutput simulationOutput;
	private Directory directory = new Directory(this);
	private IMap map;
	private int time;
	private int endOfTheWorldTime = -1;
	private final HashMap<String, NamedLocation> namedLocations = new HashMap<String, NamedLocation>();
	private final Set<Object> objects = new HashSet<Object>();

	/**
	 * Creates a world with a specific simulation output
	 * 
	 * @param so
	 */
	public World(SimulationOutput simulationOutput) {
		this.simulationOutput = simulationOutput;
	}

	public void addNamedLocation(NamedLocation nl) {
		namedLocations.put(nl.getName(), nl);
	}

	/**
	 * Adds an object to the world
	 * 
	 * @param o
	 */
	public void addObject(Object o) {
		objects.add(o);
	}

	/**
	 * Returns true if this is the last simulation cycle.
	 * 
	 * @return
	 */
	public boolean endOfTheWorld() {
		return time == endOfTheWorldTime;
	}

	/**
	 * @return Returns the directory.
	 */
	public Directory getDirectory() {
		return directory;
	}

	public int getEndOfTheWorldTime() {
		return endOfTheWorldTime;
	}

	/**
	 * @return Returns the map.
	 */
	public IMap getMap() {
		return map;
	}

	public NamedLocation getNamedLocation(String name) {
		return namedLocations.get(name);
	}

	/**
	 * Returns the objects of type c
	 * 
	 * @param c
	 * @return
	 */
	@SuppressWarnings("unchecked")
	public <X> List<X> getObjectsOfType(Class<X> c) {
		final List<X> retval = new ArrayList<X>();
		for (final Object a : objects) {
			if (a.getClass().isAssignableFrom(c)) {
				retval.add((X) a);
			}
		}
		return retval;
	}

	public SimulationOutput getSimulationOutput() {
		return simulationOutput;
	}

	/**
	 * @return Returns the time.
	 */
	public int getTime() {
		return time;
	}

	/**
	 * @param directory
	 *            The directory to set.
	 */
	public void setDirectory(Directory directory) {
		this.directory = directory;
	}

	public void setEndOfTheWorldTime(int endOfTheWorldTime) {
		this.endOfTheWorldTime = endOfTheWorldTime;
	}

	/**
	 * @param map
	 *            The map to set.
	 */
	public void setMap(IMap map) {
		this.map = map;
	}

	/**
	 * @param time
	 *            The time to set. Sets the time, it also calls update on the
	 *            agents in the directory.
	 */
	public void setTime(int time) {
		this.time = time;
	}
}
