package yaes.framework.world.map;

import yaes.framework.entity.control.ControlException;

/**
 * An abstract implementation of the concept of a moving object
 * 
 * @author Lotzi Boloni
 * 
 */
public abstract class AbstractMovingObject implements IMovingObject {
	private Location location;
	private Speed speed;
	private Acceleration acceleration;
	private double updateTime;

	/**
	 * Constructor. Empty location.
	 * 
	 * @param location
	 */
	public AbstractMovingObject() {
		this.location = new Location(0, 0);
		this.speed = new Speed(0, 0);
		this.acceleration = new Acceleration(0, 0);
		this.updateTime = 0;
	}

	/**
	 * Constructor with the setting of the initial location
	 * 
	 * @param location
	 */
	public AbstractMovingObject(Location location) {
		this.location = location;
		this.speed = new Speed(0, 0);
		this.acceleration = new Acceleration(0, 0);
		this.updateTime = 0;
	}

	/**
	 * Constructor with the setting of the initial location and update time
	 * 
	 * @param location
	 */
	public AbstractMovingObject(Location location, double updateTime) {
		this.location = location;
		this.speed = new Speed(0, 0);
		this.acceleration = new Acceleration(0, 0);
		this.updateTime = updateTime;
	}

	public Acceleration getAcceleration() {
		return acceleration;
	}

	/**
	 * Returns the location of the node.
	 */
	public Location getLocation() {
		return location;
	}

	public Speed getSpeed() {
		return speed;
	}

	public double getUpdateTime() {
		return updateTime;
	}

	public void setAcceleration(Acceleration acceleration) {
		this.acceleration = acceleration;
	}

	public void setLocation(Location location) {
		this.location = location;
	}

	public void setSpeed(Speed speed) {
		this.speed = speed;
	}

	public void setUpdateTime(double updateTime) {
		this.updateTime = updateTime;
	}

	/**
	 * Implements the normal laws of motion. It needs to be called at every
	 * update
	 * 
	 * @param time
	 * @throws ControlException
	 */
	public void updateLocation(double time) throws ControlException {
		final double deltaTime = time - updateTime;
		updateTime = time;
		if (speed != null) {
			final double xPos = location.getX() + speed.getXSpeed() * deltaTime;
			final double yPos = location.getY() + speed.getYSpeed() * deltaTime;
			// location = new Location(xPos, yPos);
			setLocation(new Location(xPos, yPos));
		}
		// double newXSpeed = speed.getXSpeed() +
		// acceleration.getXAcceleration() * deltaTime;
		// double newYSpeed = speed.getYSpeed() +
		// acceleration.getYAcceleration() * deltaTime;
		// speed = new Speed(newXSpeed, newYSpeed);
	}
}
