/*
 * Created on Sep 9, 2004
 *
 * To change the template for this generated file go to
 * Window - Preferences - Java - Code Generation - Code and Comments
 */
package yaes.framework.world.map;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Random;

import yaes.ui.text.TextUi;

/**
 * This object implements a path
 * 
 * @author Lotzi Boloni
 * 
 */
public class Path implements Serializable {
	private static final long serialVersionUID = 3616733768275211312L;
	private final List<Location> path;

	public Path() {
		path = new ArrayList<Location>();
	}

	/**
	 * Adds a location to the end of the path
	 * 
	 * @param location
	 */
	public synchronized void addLocation(Location location) {
		path.add(location);
	}

	public boolean containsLocation(Location loc) {
		return path.contains(loc);
	}

	public boolean containsLocationValue(Location loc) {
		for (final Location location : path) {
			if (location.equals(loc)) {
				return true;
			}
		}
		return false;
	}

	/**
	 * This should have never been put here.
	 * 
	 * @deprecated
	 * @param map
	 * @return
	 */
	@Deprecated
	public Location getLastUnsafeLocation(IMap map) {
		final WhiteIsAccessible checker = new WhiteIsAccessible();
		int index = 0;
		Location loc = path.get(index);
		for (index = path.size() - 1; index > 0; index--) {
			loc = path.get(index);
			if (!checker.isAccessible(map, loc)) {
				return path.get(index + 1);
			}
		}
		loc = path.get(path.size() - 1);
		return loc;
	}

	public Location getLocationAt(int index) {
		return path.get(index);
	}

	/**
	 * Returns the path as a list of locations
	 * 
	 * @return
	 */
	public List<Location> getLocationList() {
		return path;
	}

	public Location getNextLocation(Location loc, int index) {
		int locIndex = -1;
		locIndex = indexOfLocation(loc);
		if ((locIndex >= 0) && (locIndex + index < getPathSize())
				&& (locIndex + index > 0)) {
			return getLocationAt(locIndex + index);
		}
		return null;
	}

	public int getPathSize() {
		return path.size();
	}

	/**
	 * Returns a random location on the path
	 * 
	 * @param random
	 *            the random generator to be used
	 * @return
	 */
	public Location getRandomLocation(Random random) {
		final int pathSize = path.size() - 1;
		if (pathSize < 0) {
			return null;
		}
		final int randomLocation = random.nextInt(pathSize);
		return getLocationAt(randomLocation);
	}

	public Path getReversePath() {
		final Path rPath = new Path();
		int i = 0;
		for (i = path.size() - 1; i >= 0; i--) {
			rPath.addLocation(path.get(i));
		}
		return rPath;
	}

	public Path getSubPath(Location fromLocation, Location toLocation) {
		Path subPath = null;
		subPath = new Path();
		boolean startPath = false;
		if (containsLocation(fromLocation) && containsLocation(toLocation)) {
			for (int i = 0; i < path.size(); i++) {
				if (path.get(i).equals(fromLocation)) {
					startPath = true;
				}
				if (startPath) {
					subPath.addLocation(path.get(i));
				}
				if (path.get(i).equals(toLocation)) {
					startPath = false;
					break;
				}
			}
		}
		return subPath;
	}

	public double getTimetoReachLocation(Location from, Location to,
			double speed) {
		double time = -1.0;
		final Path subPath = getSubPath(from, to);
		if (subPath.getPathSize() == 0) {
			TextUi
					.println("warning: no path could be found between given locations. Time to traverse it can not be computed");
			return Double.MAX_VALUE;
		}
		final PathLength pathCost = new PathLength();
		final double distance = pathCost.cost(subPath);
		time = distance / speed;
		TextUi.println("INFO: distance:" + distance + " speed:" + speed
				+ " time:" + time);
		return time;
	}

	public int indexOfLocation(Location loc) {
		return path.indexOf(loc);
	}

	@Override
	public String toString() {
		String temp = "";
		for (final Location element : getLocationList()) {
			temp += element;
		}
		return temp;
	}
}
