package yaes.ui.propertysheet;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.util.ArrayList;
import java.util.logging.Logger;

import javax.swing.BoxLayout;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.border.Border;
import javax.swing.border.EmptyBorder;

public class DialogFrame extends JDialog implements ErrorPresenter,
		ActionListener {
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	public static final int BUTTON_HGAP = 10;
	public static final int BUTTON_VGAP = 0;
	public static final int BORDERWIDTH = 10;
	private static final String ICON_OK = "com/cplane/ic/gui/resources/images/ok.png";
	private static final String ICON_CANCEL = "com/cplane/ic/gui/resources/images/cancel.png";
	public static final String NAME_OK = "OK";
	public static final String NAME_CANCEL = "Cancel";
	// private final String textOK = DialogFrame.NAME_OK;
	// private final String textCancel = DialogFrame.NAME_CANCEL;
	public JPanel theWidget;
	private boolean success;
	private ActionListener actionListener = null;
	// private StatusPanel statusPanel;
	// private final Component parent;
	private JButton okButton;
	private JButton cancelButton;
	private final ArrayList<JButton> buttons = new ArrayList<JButton>();
	private JPanel theButtonPanel;
	private boolean actionPerformedFailed = false;
	int deltaX;
	int deltaY;

	public DialogFrame(JDialog parent, JPanel theWidget) {
		super(parent);
		// this.parent = parent;
		init(theWidget);
	}

	/**
	 * Constructor
	 */
	public DialogFrame(JFrame parent, JPanel theWidget) {
		super(parent);
		// this.parent = parent;
		init(theWidget);
	}

	public void actionPerformed(ActionEvent e) {
		final String name = ((Component) e.getSource()).getName();
		/**
		 * if (name.equals("Pin")) { JComponent pane =
		 * MainApp.getInstance().propertyPane; if(pane instanceof PinBoard) {
		 * PinBoard pinBoard = (PinBoard)pane; pinBoard.pin(theWidget); }
		 * hide(); dispose(); }
		 */
		if (theWidget instanceof ActionListener) {
			((ActionListener) theWidget).actionPerformed(e);
		} else if (actionListener != null) {
			actionListener.actionPerformed(e);
		}
		if (name.equals(DialogFrame.NAME_CANCEL)) {
			success = false;
			setVisible(false);
			dispose();
		}
		if (name.equals(DialogFrame.NAME_OK)) {
			// keep dialog open if action fails (user has to correct or to
			// cancel)
			if (actionPerformedFailed) {
				// for next attempt
				actionPerformedFailed = false;
				return;
			}
			success = true;
			setVisible(false);
			dispose();
		}
	}

	public void addButton(String buttonName, String buttonText,
			String buttonIcon) {
		final JButton button = new JButton(buttonText);
		button.addActionListener(this);
		button.setName(buttonName);
		if ((buttonIcon != null) && (buttonIcon.length() > 0)) {
			button.setIcon(new ImageIcon(buttonIcon));
		}
		final Dimension dim = cancelButton.getPreferredSize();
		button.setPreferredSize(new Dimension(dim.width, dim.height));
		buttons.add(button);
	}

	public boolean execModalDialog() {
		for (final JButton button : buttons) {
			theButtonPanel.add(button);
		}
		pack();
		this.setModal(true);
		DialogHelper.setDialogLocation(this);
		this.setVisible(true);
		return success;
	}

	public int getUsersYesNoCancelOption(String message) {
		return JOptionPane.showConfirmDialog(this, message, "Warning",
				JOptionPane.YES_NO_CANCEL_OPTION,
				JOptionPane.INFORMATION_MESSAGE);
	}

	/**
	 * Dialog to display message to the user
	 * 
	 * @param message
	 */
	public boolean getUsersYesNoOption(String message) {
		if (JOptionPane.showConfirmDialog(this, message, "Warning",
				JOptionPane.YES_NO_OPTION, JOptionPane.INFORMATION_MESSAGE) == JOptionPane.YES_OPTION) {
			return true;
		}
		return false;
	}

	private void init(JPanel theWidget) {
		this.theWidget = theWidget;
		this.addComponentListener(new ComponentAdapter() {
			@Override
			public void componentResized(ComponentEvent e) {
				windowResized();
			}
		});
		// if(theWidget instanceof Pinnable)
		// this.isPinnable = true;
		getContentPane().setLayout(new BorderLayout());
		final JPanel theBorderPanel = new JPanel();
		Border border = new EmptyBorder(DialogFrame.BORDERWIDTH,
				DialogFrame.BORDERWIDTH, DialogFrame.BORDERWIDTH,
				DialogFrame.BORDERWIDTH);
		theBorderPanel.setBorder(border);
		theBorderPanel
				.setLayout(new BoxLayout(theBorderPanel, BoxLayout.X_AXIS));
		theBorderPanel.add(theWidget);
		final JPanel outerPanel = new JPanel(new BorderLayout());
		getContentPane().add(outerPanel, BorderLayout.CENTER);
		theButtonPanel = new JPanel();
		final FlowLayout fl = (FlowLayout) theButtonPanel.getLayout();
		fl.setHgap(DialogFrame.BUTTON_HGAP);
		fl.setVgap(DialogFrame.BUTTON_VGAP);
		fl.setAlignment(FlowLayout.RIGHT);
		border = new EmptyBorder(0, 0, DialogFrame.BORDERWIDTH, 0);
		theButtonPanel.setBorder(border);
		// the OK button
		okButton = new JButton(DialogFrame.NAME_OK);
		okButton.addActionListener(this);
		okButton.setName(DialogFrame.NAME_OK);
		okButton.setIcon(new ImageIcon(DialogFrame.ICON_OK));
		buttons.add(okButton);
		getRootPane().setDefaultButton(okButton);
		// the Cancel button
		cancelButton = new JButton(DialogFrame.NAME_CANCEL);
		cancelButton.addActionListener(this);
		cancelButton.setName(DialogFrame.NAME_CANCEL);
		cancelButton.setIcon(new ImageIcon(DialogFrame.ICON_CANCEL));
		final Dimension dim = cancelButton.getPreferredSize();
		okButton.setPreferredSize(new Dimension(dim.width, dim.height));
		buttons.add(cancelButton);
		outerPanel.add(theBorderPanel, BorderLayout.CENTER);
		outerPanel.add(theButtonPanel, BorderLayout.SOUTH);
		/*
		 * if(isPinnable) { statusPanel = new StatusPanel(StatusPanel.PIN);
		 * statusPanel.getPinButton().addActionListener(this);
		 * getContentPane().add(statusPanel,BorderLayout.SOUTH); }
		 */
		pack();
	}

	public void moveButtonToPosition(String buttonName, int position) {
		for (JButton button : buttons) {
			if (button.getName().equals(buttonName)) {
				buttons.remove(button);
				buttons.add(position, button);
				return;
			}
		}
	}

	public void setActionListener(ActionListener listener) {
		this.actionListener = listener;
	}

	public void setActionPerformedFailed(boolean actionFailed) {
		actionPerformedFailed = actionFailed;
	}

	public void setButtonEnabled(String buttonName, boolean enabled) {
		for (JButton button : buttons) {
			if (button.getName().equals(buttonName)) {
				button.setEnabled(enabled);
				return;
			}
		}
	}

	public void setButtonText(String buttonName, String text) {
		for (JButton button : buttons) {
			if (button.getName().equals(buttonName)) {
				button.setText(text);
				return;
			}
		}
	}

	public void setButtonVisible(String buttonName, boolean visible) {
		for (JButton button : buttons) {
			if (button.getName().equals(buttonName)) {
				button.setVisible(visible);
				return;
			}
		}
	}

	public void setCancelButtonEnabled(boolean enabled) {
		cancelButton.setEnabled(enabled);
	}

	public void setCancelButtonText(String text) {
		cancelButton.setText(text);
	}

	public void setOkButtonEnabled(boolean enabled) {
		okButton.setEnabled(enabled);
	}

	public void setOkButtonText(String text) {
		okButton.setText(text);
	}

	/**
	 * Dialog to display error message to the user
	 * 
	 * @param message
	 */
	public void showError(String message) {
		JOptionPane.showMessageDialog(this, message, "Error!",
				JOptionPane.ERROR_MESSAGE, null);
	}

	/**
	 * Dialog to display message to the user
	 * 
	 * @param message
	 */
	public void showMessage(String message) {
		JOptionPane.showMessageDialog(this, message, "Information!",
				JOptionPane.INFORMATION_MESSAGE, null);
	}

	/**
	 * Dialog to display message without interrupting the user
	 * 
	 * @param message
	 */
	public void showSilentMessage(String errorText) {
		Logger.getLogger("avantgarde.gui").severe(
				"showSilentMessage not implemented");
	}

	private void windowResized() {
		if (theWidget.getPreferredSize() == theWidget.getMinimumSize()) {
			final Dimension minSize = getPreferredSize();
			int width = getSize().width;
			int height = getSize().height;
			if ((width < minSize.width) || (height < minSize.height)) {
				if (width <= minSize.width) {
					width = minSize.width;
				}
				if (height <= minSize.height) {
					height = minSize.height;
				}
				setSize(new Dimension(width, height));
			}
		}
	}
}
