package yaes.ui.propertysheet;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Container;
import java.awt.ContainerOrderFocusTraversalPolicy;
import java.awt.DefaultKeyboardFocusManager;
import java.awt.Font;
import java.awt.KeyboardFocusManager;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.InputEvent;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.beans.FeatureDescriptor;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyEditor;
import java.lang.reflect.InvocationTargetException;
import java.util.Iterator;
import java.util.Vector;

import javax.swing.DefaultComboBoxModel;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JRootPane;
import javax.swing.JTextField;
import javax.swing.SwingUtilities;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import yaes.ui.propertysheet.editors.ChoicePropertyEditor;
import yaes.ui.propertysheet.editors.CustomEditorPresenter;
import yaes.ui.propertysheet.editors.CustomEditorWidget;

/**
 * Visual Java Bean for editing of properties. It takes the model and represents
 * the property editor for it.
 * 
 */
public class PropertyPanel extends JComponent {
	//
	// *******************
	//
	/** Listener on <code>customizeButton</code>. */
	private final class CustomizeListener implements SheetButtonListener {
		/**
		 * Invoked when the mouse exits a component. Dummy implementation. Does
		 * nothing.
		 */
		protected Vector<?> changeListeners = null;

		private void notifyChangeListeners() {
			if (changeListeners == null) {
				return;
			}
			final ChangeEvent e = new ChangeEvent(this);
			for (final Iterator<?> it = changeListeners.iterator(); it
					.hasNext();) {
				((ChangeListener) it.next()).stateChanged(e);
			}
		}

		public void setChangeListeners(Vector<?> changeListeners) {
			this.changeListeners = changeListeners;
		}

		/**
		 * Invoked when the mouse has been clicked on a component.
		 */
		public void sheetButtonClicked(ActionEvent e) {
			final JRootPane pane = ((JComponent) e.getSource()).getRootPane();
			final Component comp = pane.getParent();
			final Component ce = editor.getCustomEditor();
			if (ce instanceof Window) {
				final Window w = (Window) ce;
				w.pack();
				w.setVisible(true);
				return;
			}
			if (ce instanceof CustomEditorPresenter) {
				((CustomEditorPresenter) ce).showDialog(comp);
				updateModel(editor.getValue());
				notifyChangeListeners();
			}
			// display custom dialog here
			// Fixme add app Frame here
			if (ce instanceof JPanel) {
				final Object oldValue = editor.getAsText();
				Object newValue = null;
				DialogFrame dlg = null;
				if (comp instanceof JDialog) {
					dlg = new DialogFrame((JDialog) comp, (JPanel) ce);
				} else if (comp instanceof JFrame) {
					dlg = new DialogFrame((JFrame) comp.getParent(),
							(JPanel) ce);
				}
				if (dlg != null) {
					final PropDialogActionListener actionListener = new PropDialogActionListener(
							ce, model);
					actionListener.setChangeListeners(changeListeners);
					dlg.setActionListener(actionListener);
					if (ce instanceof CustomEditorWidget) {
						final CustomEditorWidget widget = (CustomEditorWidget) ce;
						dlg.setTitle(widget.getWidgetTitle());
						widget.setErrorPresenter(dlg);
					}
					dlg.execModalDialog();
					newValue = editor.getAsText();
					// test if newValue is not equal to oldValue
					if (((newValue != null) && !newValue.equals(oldValue))
							|| ((newValue == null) && (oldValue != null))) {
						notifyChangeListeners();
					}
				}
			}
			customDialogShown = true;
			// update editor from the model
			if (canRead) {
				try {
					ignoreEvents = true;
					final Object newValue = model.getValue();
					final Object oldValue = editor.getValue();
					// test if newValue is not equal to oldValue
					if (((newValue != null) && !newValue.equals(oldValue))
							|| ((newValue == null) && (oldValue != null))) {
						editor.setValue(newValue);
						notifyChangeListeners();
					}
				} /*
					 * catch (ProxyNode.DifferentValuesException dve) {
					 * differentValues = true; }
					 */
				catch (final Exception ite) {
					processThrowable(ite);
				} finally {
					ignoreEvents = false;
				}
			}
			reset();
			requestFocus();
			customDialogShown = false;
		}

		/**
		 * Invoked when the mouse enters a component. Dummy implementation. Does
		 * nothing.
		 */
		public void sheetButtonEntered(ActionEvent e) {
		}

		public void sheetButtonExited(ActionEvent e) {
		}

		private void updateModel(Object value) {
			try {
				model.setValue(value);
				notifyChangeListeners();
			} catch (final java.lang.reflect.InvocationTargetException ite) {
				return;
			} catch (final IllegalStateException ise) {
				return;
			}
		}
	} // End of class CustomizeListener.

	//
	// *******************
	//
	/**
	 * Property change listener for the editor.
	 */
	private class EditorListener implements PropertyChangeListener {
		protected Vector<?> changeListeners = new Vector();

		private void notifyChangeListeners() {
			if (changeListeners == null) {
				return;
			}
			final ChangeEvent e = new ChangeEvent(this);
			for (final Iterator<?> it = changeListeners.iterator(); it
					.hasNext();) {
				((ChangeListener) it.next()).stateChanged(e);
			}
		}

		/** Property was changed. */
		public void propertyChange(PropertyChangeEvent evt) {
			if (ignoreEvents) {
				return;
			}
			if (!isWriteState && !customDialogShown && (preferences < 2)) {
				return;
			}
			if (customDialogShown) {
				return;
			}
			// ExPropertyEditor can fire PROP_VALUE_VALID property change, which
			// doesn't mean the change of edited property.
			/*
			 * if (ExPropertyEditor.PROP_VALUE_VALID.equals (evt.getPropertyName
			 * ())) { return; }
			 */
			if (editor != null) {
				try {
					ignoreEvents = true;
					final Object newValue = editor.getValue();
					Object oldValue = null;
					try {
						if (canRead) {
							oldValue = model.getValue();
						}
						// notifyChangeListeners();
					} catch (final Exception e) {
					}
					// test if newValue is not equal to oldValue
					if (((newValue != null) && !newValue.equals(oldValue))
							|| ((newValue == null) && (oldValue != null))) {
						model.setValue(newValue);
						notifyChangeListeners();
					}
				} catch (final InvocationTargetException e) {
					try {
						if (canRead) {
							editor.setValue(model.getValue());
							notifyChangeListeners();
						}
					} catch (final Exception ex) {
						// PropertyDialogManager.notify(ex);
					}
				} finally {
					ignoreEvents = false;
				}
			}
		}

		public void setChangeListeners(Vector<?> changeListeners) {
			this.changeListeners = changeListeners;
		}
	} // End of class EditorListener.

	//
	// ******************* innerclasses
	//
	/** Gets localized string from specified key. */
	/** Empty implementation of the <code>PropertyModel</code> interface. */
	private static class EmptyModel implements PropertyModel {
		/** @return <code>null</code> */
		public Vector<?> changeListeners = null;

		/** Dummy implementation. Does nothing. */
		public void addPropertyChangeListener(PropertyChangeListener l) {
		}

		public Vector getChangeListeners() {
			return changeListeners;
		}

		/** @return <code>null</code> */
		@SuppressWarnings("unchecked")
		public Class getPropertyEditorClass() {
			return null;
		}

		/** @return <code>Object.class</code> */
		@SuppressWarnings("unchecked")
		public Class getPropertyType() {
			return Object.class;
		}

		public String[] getTags() {
			return null;
		}

		public Object getValue() throws InvocationTargetException {
			return null;
		}

		public boolean isPropertyName() {
			return true;
		}

		public boolean isValueReadOnly() {
			return false;
		}

		/** Dummy implementation. Does nothing. */
		public void removePropertyChangeListener(PropertyChangeListener l) {
		}

		public void setChangeListeners(Vector<PropertyChangeListener> v) {
			changeListeners = v;
		}

		/** Dummy implementation. Does nothing. */
		public void setValue(Object v) throws InvocationTargetException {
		}
	} // End of class EmptyModel.

	//
	// *******************
	//
	/**
	 * Property change listener for the model.
	 */
	private class ModelListener implements PropertyChangeListener {
		/** Property was changed. */
		public void propertyChange(PropertyChangeEvent evt) {
			if (PropertyModel.PROP_VALUE.equals(evt.getPropertyName())) {
				if (editor != null) {
					Mutex.EVENT.readAccess(new Runnable() {
						public void run() {
							try {
								ignoreEvents = true;
								differentValues = false;
								final Object newValue = model.getValue();
								final Object oldValue = editor.getValue();
								// test if newValue is not equal to oldValue
								if (((newValue != null) && !newValue
										.equals(oldValue))
										|| ((newValue == null) && (oldValue != null))) {
									editor.setValue(newValue);
								}
								refresh();
							} catch (final InvocationTargetException e) {
							} finally {
								ignoreEvents = false;
							}
						}
					});
				}
			}
		}
	} // End of class ModelListener.

	//
	// *******************
	//
	class PropDialogActionListener implements ActionListener {
		private final Component component;
		private final PropertyModel model;
		protected Vector<?> changeListeners = null;

		public PropDialogActionListener(Component component, PropertyModel model) {
			this.component = component;
			this.model = model;
		}

		public void actionPerformed(ActionEvent e) {
			final String name = ((Component) e.getSource()).getName();
			Object oldValue = null;
			try {
				oldValue = model.getValue();
			} catch (final Exception exc) {
				// Ignored, there can be number of exceptions
				// when asking for old values...
			}
			if (name.equals(DialogFrame.NAME_CANCEL)) {
				cancelValue(oldValue);
			}
			if (name.equals(DialogFrame.NAME_OK)) {
				if (component instanceof EnhancedCustomPropertyEditor) {
					try {
						final Object newValue = ((EnhancedCustomPropertyEditor) component)
								.getPropertyValue();
						model.setValue(newValue);
						notifyChangeListeners();
					} catch (final java.lang.reflect.InvocationTargetException ite) {
						return;
					} catch (final IllegalStateException ise) {
						return;
					}
				} else {
					if (component instanceof CustomEditorWidget) {
						((CustomEditorWidget) component).actionOk();
					}
					final Object lastValueFromEditor = editor.getValue();
					try {
						model.setValue(lastValueFromEditor);
						notifyChangeListeners();
					} catch (final java.lang.reflect.InvocationTargetException ite) {
						return;
					} catch (final IllegalStateException ise) {
						return;
					}
				}
				// this is an old hack allowing to notify a cached value
				// obtained via propertyChangeEvent from the editor
			}
		}

		private void cancelValue(Object oldValue) {
			try {
				model.setValue(oldValue);
			} catch (final Exception e) {
				// Ignored, there can be number of exceptions
				// when asking for old values...
			}
		}

		private void notifyChangeListeners() {
			if (changeListeners == null) {
				return;
			}
			final ChangeEvent e = new ChangeEvent(this);
			for (final Iterator<?> it = changeListeners.iterator(); it
					.hasNext();) {
				((ChangeListener) it.next()).stateChanged(e);
			}
		}

		public void setChangeListeners(Vector<?> changeListeners) {
			this.changeListeners = changeListeners;
		}
	} // end of PropDialogActionListener

	//
	// *******************
	//
	/** Listener on <code>readComponent</code>. */
	private final class ReadComponentListener implements SheetButtonListener {
		/**
		 * Invoked when the mouse has been clicked on a component.
		 */
		public void sheetButtonClicked(ActionEvent e) {
			if (SheetButton.RIGHT_MOUSE_COMMAND.equals(e.getActionCommand())
					|| isWriteState) {
				return;
			}
			SwingUtilities.invokeLater(new Runnable() {
				public void run() {
					setWriteState();
				}
			});
		}

		/**
		 * Invoked when the mouse enters a component.
		 */
		public void sheetButtonEntered(ActionEvent e) {
		}

		/**
		 * Invoked when the mouse exits a component.
		 */
		public void sheetButtonExited(ActionEvent e) {
		}
	} // End of class ReadComponentListener.

	//
	// *******************
	//
	/**
	 * Implementation of the <code>PropertyModel</code> interface keeping a
	 * <code>Node.Property</code>.
	 */
	/**
	 * Listener on textField(<code>JTextField</code>) or comboBox(<code>JComboBox</code>) -
	 * input line components controlled by this panel.
	 */
	private final class WriteComponentListener extends KeyAdapter implements
			ActionListener, FocusListener {
		/** Holds old value. */
		private Object oldValue;
		/** Task which handles action performed from combo box. */
		private Runnable comboActionTask;
		/**
		 * Task which requests default focus for enclosing class when setting to
		 * 'readState'.
		 */
		private Runnable requestFocusTask;
		protected Vector<?> changeListeners = null;

		/** Implements <code>ActionListener</code> interface. */
		public void actionPerformed(ActionEvent e) {
			if (!isWriteState) {
				return;
			}
			if (e.getSource() == comboBox) {
				// XXX #16101 - There is a problem with actions fired
				// from combo, we can't see the diff if it was caused by mouse
				// selection in popup or key navigation in popup, thus we plan
				// to handle this event later so we will know to decide what
				// was the cause of the action depening on it if popup is
				// visible or not (yes->mouse, no->key navigation).
				SwingUtilities.invokeLater(getComboActionTask());
			} else {
				if (e.getSource() == textField) {
					final String val = textField.getText();
					changeValue(val);
				}
				prepareReadState();
			}
		}

		/**
		 * Changes value if needed.
		 */
		public void changeValue(String s) {
			if ((s != null) && ((editor.getValue() == null // Fix #13339
					) || !s.equals(editor.getAsText()))) {
				if (!setAsText(s)) {
					resetOldValue();
				}
			}
		}

		/** Implements <code>FocusListener</code> interface. */
		public void focusGained(FocusEvent e) {
			if (e.getSource() == textField) {
				textField.selectAll();
			} else if ((comboBox != null) && comboBox.isEditable()) {
				comboBox.getEditor().selectAll();
			}
		}

		/** Implements <code>FocusListener</code> interface. */
		public void focusLost(final FocusEvent e) {
			if (!isWriteState) {
				return;
			}
			if ((comboBox != null) && (e != null) && e.isTemporary()) {
				return;
			}
			final boolean supportsCustom = (editor != null)
					&& editor.supportsCustomEditor();
			if (/* supportsCustom || */comboBox != null) {
				// XXX
				// In this case we need to find out if the focus was
				// moved to component inside this PropetyPanel hierarchy,
				// currently to "..." customize button.
				// To figure out the new focus owner we have to skip the current
				// task.
				SwingUtilities.invokeLater(new Runnable() {
					public void run() {
						// Reset 'read state' only in case the focus was lost
						// from this PropertyPanel hierarchy.
						final KeyboardFocusManager focusManager = new DefaultKeyboardFocusManager();
						if ((focusManager.getFocusOwner() == null)
								&& isWriteState) {
							resetReadState(e);
						}
					}
				});
			} else {
				resetReadState(e);
			}
		}

		/** Gets <code>comboActionTask</code>. */
		private synchronized Runnable getComboActionTask() {
			if (comboActionTask == null) {
				comboActionTask = new Runnable() {
					public void run() {
						// XXX For combos allow key navigation in popup.
						if (comboBox.isPopupVisible() || !isWriteState) {
							return;
						}
						changeValue((String) comboBox.getSelectedItem());
						notifyChangeListeners();
						prepareReadState();
					}
				};
			}
			return comboActionTask;
		}

		/** Gets <code>requestFocusTask</code>. */
		private synchronized Runnable getRequestFocusTask() {
			if (requestFocusTask == null) {
				requestFocusTask = new Runnable() {
					public void run() {
						final Component focused = PropertyPanel
								.getDefaultFocusComponent(PropertyPanel.this);
						if (focused != null) {
							focused.requestFocus();
						}
					}
				};
			}
			return requestFocusTask;
		}

		private boolean isClickCustomizeButtonEvent(KeyEvent e) {
			return (e.getKeyCode() == PropertyPanel.ClickCustomizeButtonKeyCode)
					&& (e.getModifiers() == PropertyPanel.ClickCustomizeButtonModifier) ? true
					: false;
		}

		/** Overrides <code>KeyAdapter</code> superclass method. */
		@Override
		public void keyPressed(KeyEvent e) {
			final Object source = e.getSource();
			if (e.getKeyCode() == KeyEvent.VK_TAB) {
				selectionLost(e);
			}
			if (e.getKeyCode() == KeyEvent.VK_ESCAPE) {
				if (// XXX ESC during navigation in popup resets the old value.
				(comboBox != null)
						&& comboBox.isPopupVisible()
						&& (source instanceof Component)
						&& SwingUtilities.isDescendingFrom((Component) source,
								comboBox)) {
					resetOldValue();
				} else if (source == textField) {
					resetOldValue();
				}
				prepareReadState();
			} else if ((e.getKeyCode() == KeyEvent.VK_ENTER)
					&& (comboBox != null)
					&& (source instanceof Component)
					&& SwingUtilities.isDescendingFrom((Component) source,
							comboBox) && !comboBox.isEditable()
					&& ((preferences & PropertyPanel.PREF_INPUT_STATE) == 0)) {
				changeValue((String) comboBox.getSelectedItem());
				prepareReadState();
			} else if (isClickCustomizeButtonEvent(e)
					&& (customizeButton != null)) {
				customizeButton
						.notifySheetButtonListenersAboutClick(new ActionEvent(
								customizeButton, ActionEvent.ACTION_FIRST + 1,
								"click"));
			}
		}

		private void notifyChangeListeners() {
			if (changeListeners == null) {
				return;
			}
			final ChangeEvent e = new ChangeEvent(this);
			for (final Iterator<?> it = changeListeners.iterator(); it
					.hasNext();) {
				((ChangeListener) it.next()).stateChanged(e);
			}
		}

		/**
		 * Prepares 'readState'. Sets 'readState' and requests default focus for
		 * changed component tree of enclosing <code>PropertyPanel</code>.
		 */
		private void prepareReadState() {
			final KeyboardFocusManager focusManager = new DefaultKeyboardFocusManager();
			final boolean hasFocus = focusManager.getFocusOwner() != null;
			setReadState();
			if (hasFocus) {
				// XXX direct call doesn't work - #16052.
				SwingUtilities.invokeLater(getRequestFocusTask());
			}
		}

		/** Resets specified value to <code>editor</code>. */
		private void resetOldValue() {
			try {
				editor.setValue(oldValue);
			} catch (final IllegalArgumentException iae) {
				// notifyExceptionWhileSettingProperty(iae);
			} catch (final Throwable throwable) {
				processThrowable(throwable);
			}
		}

		/** Resets 'read state' on focus lost change. Helper method. */
		private void resetReadState(FocusEvent e) {
			if (e.getSource() == textField) {
				changeValue(textField.getText());
			} else if ((comboBox != null) && comboBox.isEditable()) {
				changeValue((String) comboBox.getEditor().getItem());
			}
			setReadState();
		}

		private void resetReadState(KeyEvent e) {
			if (e.getSource() == textField) {
				changeValue(textField.getText());
			} else if ((comboBox != null) && comboBox.isEditable()) {
				changeValue((String) comboBox.getEditor().getItem());
			}
			setReadState();
		}

		public void selectionLost(final KeyEvent e) {
			if (!isWriteState) {
				return;
			}
			final boolean supportsCustom = (editor != null)
					&& editor.supportsCustomEditor();
			if (/* supportsCustom || */comboBox != null) {
				// XXX
				// In this case we need to find out if the focus was
				// moved to component inside this PropetyPanel hierarchy,
				// currently to "..." customize button.
				// To figure out the new focus owner we have to skip the current
				// task.
				SwingUtilities.invokeLater(new Runnable() {
					public void run() {
						// Reset 'read state' only in case the focus was lost
						// from this PropertyPanel hierarchy.
						final KeyboardFocusManager focusManager = new DefaultKeyboardFocusManager();
						if ((focusManager.getFocusOwner() == null)
								&& isWriteState) {
							resetReadState(e);
						}
					}
				});
			} else {
				resetReadState(e);
			}
		}

		/**
		 * Sets as text.
		 * 
		 * @value <code>String</code> value which is possible to convert to
		 *        the type of property.
		 * @returns <code>true</code> if succesfull, <code>false</code>
		 *          otherwise
		 */
		private boolean setAsText(String value) {
			try {
				editor.setAsText(value);
				return true;
			} catch (final IllegalArgumentException iae) {
				// notifyExceptionWhileSettingProperty(iae);
			} catch (final Throwable throwable) {
				processThrowable(throwable);
			}
			return false;
		}

		public void setChangeListeners(Vector<?> changeListeners) {
			this.changeListeners = changeListeners;
		}

		/** Sets <code>oldValue</code>. */
		private void setOldValue(Object oldValue) {
			this.oldValue = oldValue;
		}
	} // End of class WriteComponentListener.

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	/**
	 * Constant defining preferences in displaying of value. Value should be
	 * displayed in read-only mode.
	 */
	public static final int PREF_READ_ONLY = 0x0001;
	/**
	 * Constant defining preferences in displaying of value. Value should be
	 * displayed in custom editor.
	 */
	public static final int PREF_CUSTOM_EDITOR = 0x0002;
	/**
	 * Constant defining preferences in displaying of value. Value should be
	 * displayed in editor only.
	 */
	public static final int PREF_INPUT_STATE = 0x0004;
	/** Name of the 'preferences' property. */
	public static final String PROP_PREFERENCES = "preferences"; // NOI18N
	/** Name of the 'model' property. */
	public static final String PROP_MODEL = "model"; // NOI18N
	/** Name of the read-only property 'propertyEditor'. */
	public static final String PROP_PROPERTY_EDITOR = "propertyEditor"; // NOI18N
	/** Static instance of empty PropertyModel. */
	private static final PropertyModel EMPTY_MODEL = new EmptyModel();
	/** Key code and modifier to activate Customize Button. */
	private static final int ClickCustomizeButtonKeyCode = KeyEvent.VK_E;
	private static final int ClickCustomizeButtonModifier = InputEvent.CTRL_MASK;
	/** */
	static ThreadLocal<PropertyPanel> current = new ThreadLocal<PropertyPanel>();

	/**
	 * Gets default focus component from the JComponent container hierarchy,
	 * i.e. component which calling on container requestDefaultComponent should
	 * get the focus - it differs from SwingUtilities.findFocusOwner.
	 * 
	 * @return <code>Component</code> which should get the focus as default or
	 *         <code>null</code> if there is no such one.
	 */
	private static Component getDefaultFocusComponent(JComponent container) {
		final Component[] ca = container.getComponents();
		for (Component element : ca) {
			if (element.isFocusable()) {
				return element;
			}
			if (element instanceof JComponent) {
				final Component res = PropertyPanel
						.getDefaultFocusComponent((JComponent) element);
				if (res != null) {
					return res;
				}
			}
		}
		return null;
	}

	/** Holds value of property preferences. */
	private int preferences;
	/** Holds value of property model. */
	private PropertyModel model;
	/** Current property editor. */
	private PropertyEditor editor;
	/** Descriptor for the property. */
	private final FeatureDescriptor descriptor = new FeatureDescriptor();
	/** Holds painting style. */
	private int paintingStyle;
	/** Foreground color of values. */
	private Color foregroundColor;
	/** Foreground color of disabled values. */
	private Color disabledColor;
	/** Is plastic property value. */
	private boolean plastic;
	private Color backgroundColor;
	private int fontStyle;
	/** Indicates whether the property is writable. */
	private boolean canWrite = true;
	/**
	 * In the (rare) case when the property is not readable value of this
	 * variable is set to false.
	 */
	private final boolean canRead = true;
	// private variables for visual controls
	// ...........................................
	/** Component for showing property value is stored here. */
	private SheetButton readComponent;
	/** Component cache. */
	private SheetButton textView;
	/** Component cache. */
	private PropertyShow propertyShow;
	/** Component cache. */
	private SheetButton paintView;
	/** Component cache. */
	private SheetButton customizeButton;
	/**
	 * If this is true the read component is visible and we are not performing
	 * any component switch.
	 */
	private boolean isReadState;
	/**
	 * If this is true the write component is visible and we are not performing
	 * any component switch.
	 */
	private boolean isWriteState;
	/** Prevents firing back to ourselves. */
	private boolean ignoreEvents;
	/** Set to <code>true</code> when the custom dialog is showing. */
	private boolean customDialogShown;
	/** <code>True</code> if we represent more values in this panel. */
	private boolean differentValues;
	/** TextField used for editing the property value as text. */
	private JTextField textField;

	/** Combo used for editing the property value as tags. */
	private JComboBox comboBox;

	/** Listener capable of switching to the writing state. */
	private ReadComponentListener readComponentListener;

	/**
	 * Listener on textField and comboBox - it allows to switch back to the
	 * reading state.
	 */
	private WriteComponentListener writeComponentListener;

	/** Listens on changes in the model. */
	private PropertyChangeListener modelListener;

	/** Listens on changes in the editor. */
	private PropertyChangeListener editorListener;

	/**
	 * If this is not <code>null</code> the listener is added to all newly
	 * created Sheetbuttons.
	 */
	private SheetButtonListener sheetButtonListener;

	/**
	 * If this is <code>true</code> the changes made in the property editor
	 * are immediately propagated to the value of the property (to the property
	 * model).
	 */
	private final boolean changeImmediate = true;

	protected Vector<?> changeListeners = null;

	// constructors -------------------------------------------------------
	/**
	 * Creates new PropertyPanel with the empty DefaultPropertyModel
	 */
	public PropertyPanel() {
		this(PropertyPanel.EMPTY_MODEL, 0);
	}

	/**
	 * Creates new PropertyPanel
	 * 
	 * @param model
	 *            The model for displaying
	 */
	public PropertyPanel(PropertyModel model, int preferences) {
		this.model = model;
		this.preferences = preferences;
		setLayout(new BorderLayout());
		PropertyPanel.current.set(this);
		changeListeners = model.getChangeListeners();
		final PropertySheetSettings psDefault = PropertySheetSettings
				.getDefault();
		paintingStyle = psDefault.getPropertyPaintingStyle();
		plastic = psDefault.getPlastic();
		disabledColor = psDefault.getDisabledPropertyColor();
		if (model.isPropertyName()) {
			backgroundColor = psDefault.getPropNameBkgndColor();
			foregroundColor = psDefault.getPropNameFrgndColor();
			fontStyle = psDefault.getPropNameFontStyle();
		} else {
			backgroundColor = psDefault.getPropValueBkgndColor();
			foregroundColor = psDefault.getPropValueFrgndColor();
			fontStyle = psDefault.getPropValueFontStyle();
		}
		if ((preferences & PropertyPanel.PREF_READ_ONLY) != 0) {
			canWrite = false;
		}
		model.addPropertyChangeListener(getModelListener());
		updateEditor();
		reset();
	}

	/** Adds sheet button listener to the <code>readComponent</code>. */
	void addSheetButtonListener(SheetButtonListener list) {
		this.sheetButtonListener = list;
		if (readComponent != null) {
			readComponent.addSheetButtonListener(list);
		}
	}

	/** Gets <code>customizedButton</code>, so called 'three-dot-button'. */
	private SheetButton getCustomizeButton() {
		if (customizeButton == null) {
			customizeButton = new SheetButton("...", plastic, plastic); // NOI18N
			customizeButton.setFocusable(true);
			final Font currentFont = customizeButton.getFont();
			customizeButton.setFont(new Font(currentFont.getName(), currentFont
					.getStyle()
					| Font.BOLD, currentFont.getSize()));
			customizeButton.addFocusListener(getWriteComponentListener());
			final CustomizeListener customizeListener = new CustomizeListener();
			customizeListener.setChangeListeners(changeListeners);
			customizeButton.addSheetButtonListener(customizeListener);
			// customizeButton.setToolTipText(getString("CTL_ElipsisHint"));
		}
		customizeButton.setToolTipText("To activate press Ctrl-E");
		// XXX Read-only should be handled via enabling/disabling component
		// not via settin 'active' foreground color.
		if (canWrite) {
			customizeButton.setActiveForeground(foregroundColor);
		} else {
			customizeButton.setActiveForeground(disabledColor);
		}
		return customizeButton;
	}

	/** */
	private JComponent getDisabledWriterComponent() {
		final SheetButton c = getReaderComponent();
		c.setActiveForeground(disabledColor);
		c.addFocusListener(getWriteComponentListener());
		return c;
	}

	/** init EditorListener */
	private PropertyChangeListener getEditorListener() {
		if (editorListener == null) {
			editorListener = new EditorListener();
			((EditorListener) editorListener)
					.setChangeListeners(changeListeners);
		}
		return editorListener;
	}

	/** */
	private String getExceptionString(Throwable exception) {
		if (exception instanceof InvocationTargetException) {
			exception = ((InvocationTargetException) exception)
					.getTargetException();
		}
		return "<" + exception.getClass().getName() + ">"; // NOI18N
	}

	/** Gets 'in-place' custom editor component. */
	private Component getInPlace() {
		final Component c = ((EnhancedPropertyEditor) editor)
				.getInPlaceCustomEditor();
		c.addFocusListener(getWriteComponentListener());
		c.addKeyListener(getWriteComponentListener());
		if (c instanceof JComponent) {
			((JComponent) c).setToolTipText(getPanelToolTipText());
		}
		return c;
	}

	/**
	 * This is helper method for method getInput () and getInputTag () which
	 * returns Panel with enhanced property editor open button on the "East".
	 * 
	 * @param Component
	 *            leftComponent this component will be added to the "Center" of
	 *            this panel
	 * @param boolean
	 *            existsCustomEditor is true if enhanced property editor open
	 *            button should be shown.
	 * 
	 * @return <code>JPanel</code> component
	 */
	private JComponent getInput(Component leftComponent,
			boolean existsCustomEditor) {
		JPanel panel;
		if ((leftComponent == null) && (editor != null) && editor.isPaintable()
				&& (paintingStyle != PropertySheetSettings.ALWAYS_AS_STRING)) {
			panel = new PropertyShow(editor);
		} else {
			panel = new JPanel();
		}
		panel.setLayout(new BorderLayout());
		if (leftComponent != null) {
			panel.add(leftComponent, BorderLayout.CENTER);
		}
		if (existsCustomEditor) {
			panel.add(getCustomizeButton(), BorderLayout.EAST); // NOI18N
		}
		panel.setToolTipText(getPanelToolTipText());
		panel.addFocusListener(getWriteComponentListener());
		panel.addKeyListener(getWriteComponentListener());
		return panel;
	}

	/**
	 * This is helper method for method getWriterComponent () which returns
	 * Panel with TextField in the "Center" and enhanced property editor open
	 * button on the "East". This Panel is then returned as property value
	 * editor Component.
	 * 
	 * @param String
	 *            propertyStringValue initial property value.
	 * @param boolean
	 *            editable is true if string editing should be allowed.
	 * @param boolean
	 *            existsCustomEditor is true if enhanced property editor open
	 *            button should be showen.
	 * 
	 * @return Panel Component
	 */
	private JComponent getInputLine(final String propertyStringValue,
			boolean editable) {
		textField = new JTextField();
		textField.addActionListener(getWriteComponentListener());
		textField.addKeyListener(getWriteComponentListener());
		textField.addFocusListener(getWriteComponentListener());
		textField.setText(propertyStringValue);
		textField.setEditable(editable);
		textField.setToolTipText(getPanelToolTipText());
		if (!isWriteState) {
			textField.selectAll();
		}
		return textField;
	}

	/**
	 * This is helper method for method getWriterComponent () which returns
	 * Panel with Choice in the "Center" and enhanced property editor open
	 * button on the "East". This Panel is then returned as property value
	 * editor Component.
	 * 
	 * @param tags
	 *            There are lines for Choice stored.
	 * @param selected
	 *            Line to be selected.
	 * 
	 * @return Choice Component
	 */
	private JComponent getInputTag(String[] tags, final String selected,
			boolean editable) {
		comboBox = new JComboBox();
		comboBox.setModel(new DefaultComboBoxModel(tags));
		comboBox.setMaximumRowCount(tags.length <= 12 ? tags.length : 8);
		if (selected != null) {
			for (int i = 0; i < tags.length; i++) {
				if (tags[i].equals(selected)) {
					comboBox.setSelectedIndex(i);
					break;
				}
			}
		}
		if (editable) {
			comboBox.setEditable(true);
			comboBox.setSelectedItem(selected);
			comboBox.getEditor().setItem(selected);
			comboBox.getEditor().getEditorComponent().addFocusListener(
					getWriteComponentListener());
			comboBox.getEditor().getEditorComponent().addKeyListener(
					getWriteComponentListener());
		}
		comboBox.addActionListener(getWriteComponentListener());
		comboBox.setToolTipText(getPanelToolTipText());
		return comboBox;
	}

	/**
	 * Getter for property model.
	 * 
	 * @return Value of property model.
	 */
	public PropertyModel getModel() {
		return model;
	}

	/** Lazy init of the <code>modelListener</code>. */
	private PropertyChangeListener getModelListener() {
		if (modelListener == null) {
			modelListener = new ModelListener();
		}
		return modelListener;
	}

	/**
	 * Creates SheetButton with PropertyShow representing current value of
	 * property.
	 */
	private SheetButton getPaintView() {
		if (propertyShow == null) {
			propertyShow = new PropertyShow(editor);
		} else {
			propertyShow.setEditor(editor);
		}
		paintView = new SheetButton();
		paintView.setFocusable(true);
		paintView.setBackground(backgroundColor);
		propertyShow.setBackground(backgroundColor);
		final Font font = paintView.getFont();
		paintView.setFont(new Font(font.getFontName(), fontStyle, font
				.getSize()));
		propertyShow.setFont(new Font(font.getFontName(), fontStyle, font
				.getSize()));
		// XXX Read-only should be handled via enabling/disabling component
		// not via settin 'active' foreground color.
		if (canWrite) {
			paintView.setActiveForeground(foregroundColor);
			propertyShow.setForeground(foregroundColor);
		} else {
			paintView.setActiveForeground(disabledColor);
			propertyShow.setForeground(disabledColor);
		}
		if (sheetButtonListener != null) {
			paintView.addSheetButtonListener(sheetButtonListener);
		}
		paintView.add(propertyShow);
		paintView.setPlastic(plastic);
		paintView.setToolTipText(getPanelToolTipText());
		return paintView;
	}

	/**
	 * Gets tooltip for this <code>PropertyPanel</code>.
	 * 
	 * @return tooltip retrieved from getToolTipText method or property class
	 *         type name if the former is <code>null</code> or
	 *         <code>null</code> if the model returns null as its class type
	 */
	private String getPanelToolTipText() {
		try {
			final Object value = model.getValue();
			// if (value instanceof List<String>)
			// this.setToolTipText(((List<String>) value).getText());
			this.setToolTipText(value.toString());
		} catch (final InvocationTargetException exc) {
		}
		final String toolTip = getToolTipText();
		if (toolTip != null) {
			return toolTip;
		}
		try {
			// XXX #16004, getPropertyType could return null??
			return model.getValue().toString();
		} catch (final InvocationTargetException exc) {
			return null;
		}
		/*
		 * Class clazz = model.getPropertyType(); return clazz == null ? null :
		 * clazz.getName();
		 */
	}

	/**
	 * Test whether buttons in sheet are plastic.
	 * 
	 * @return <code>true</code> if so
	 */
	boolean getPlastic() {
		return plastic;
	}

	// public methods -------------------------------------------------------
	/**
	 * Getter for property preferences.
	 * 
	 * @return Value of property preferences.
	 */
	public int getPreferences() {
		return preferences;
	}

	/**
	 * Getter for current property editor depending on the model. It could be
	 * <CODE>null</CODE> if there is not possible to obtain property editor
	 * for the current model.
	 * 
	 * @return the property editor or <CODE>null</CODE>
	 */
	public PropertyEditor getPropertyEditor() {
		return editor;
	}

	// package private methods ----------------------------------------------
	/** Return a shett button serving as readComponent if it has been initailize. */
	SheetButton getReadComponent() {
		return readComponent;
	}

	/** Lazy init of the <code>readComponentListener</code>. */
	private ReadComponentListener getReadComponentListener() {
		if (readComponentListener == null) {
			readComponentListener = new ReadComponentListener();
		}
		return readComponentListener;
	}

	// reader component ........................
	/**
	 * Creates Reader component.
	 */
	private SheetButton getReaderComponent() {
		String stringValue = null;
		SheetButton c = null;
		if (editor == null) {
			return getTextView(getTypeString(model.getPropertyType()));
		}
		try {
			if (!differentValues && canRead) {
				stringValue = editor.getAsText();
			}
			if (editor.isPaintable()
					&& ((paintingStyle == PropertySheetSettings.PAINTING_PREFERRED) || ((paintingStyle == PropertySheetSettings.STRING_PREFERRED) && (stringValue == null)))) {
				c = getPaintView();
			} else {
				c = getTextView(stringValue == null ? getTypeString(model
						.getPropertyType()) : stringValue);
			}
		} catch (final Throwable e) {
			// exception while getAsText () | isPaintable ()
			if (e instanceof ThreadDeath) {
				throw (ThreadDeath) e;
			}
			c = getTextView(getExceptionString(e));
		}
		// we should never return null (dstrupl)
		if (c == null) {
			c = getTextView("null"); // NOI18N
		}
		return c;
	}

	/**
	 * Creates SheetButton with text representing current value of property.
	 */
	private SheetButton getTextView(String str) {
		textView = new SheetButton(str, plastic, plastic);
		textView.setBackground(backgroundColor);
		final Font font = textView.getFont();
		textView
				.setFont(new Font(font.getFontName(), fontStyle, font.getSize()));
		textView.setFocusable(true);
		if (sheetButtonListener != null) {
			textView.addSheetButtonListener(sheetButtonListener);
		}
		// XXX Read-only should be handled via enabling/disabling component
		// not via settin 'active' foreground color.
		if (canWrite) {
			textView.setActiveForeground(foregroundColor);
		} else {
			textView.setActiveForeground(disabledColor);
		}
		return textView;
	}

	/** */
	@SuppressWarnings("unchecked")
	private String getTypeString(Class clazz) {
		if (clazz == null) {
			return "NoPropertyEditor";
		}
		if (clazz.isArray()) {
			return "[" + "Array Of" + " "
					+ getTypeString(clazz.getComponentType()) + "]"; // NOI18N
		}
		return "[" + clazz.getName() + "]"; // NOI18N
	}

	/** Lazy init of the <code>writeComponentListener</code>. */
	private WriteComponentListener getWriteComponentListener() {
		if (writeComponentListener == null) {
			writeComponentListener = new WriteComponentListener();
			writeComponentListener.setChangeListeners(changeListeners);
		}
		return writeComponentListener;
	}

	// writer component ........................
	/**
	 * This method returns property value editor Component like input line (if
	 * property supports setAsText (String string) method) or some others.
	 * 
	 * @return property value editor Component
	 */
	private JComponent getWriterComponent() {
		if (editor == null) {
			return getDisabledWriterComponent();
		}
		String stringValue = null;
		final boolean canEditAsText = true;
		try {
			if (!differentValues && canRead) {
				stringValue = editor.getAsText();
			}
		} catch (final Exception x) {
			processThrowable(x);
		}
		/*
		 * if ((stringValue == null) && (customEditAsText == null)){
		 * canEditAsText = false; }
		 */
		getWriteComponentListener().setOldValue(editor.getValue());
		final boolean existsCustomEditor = editor.supportsCustomEditor();
		if ((editor instanceof EnhancedPropertyEditor)
				&& ((EnhancedPropertyEditor) editor).hasInPlaceCustomEditor()) {
			return getInput(getInPlace(), existsCustomEditor);
		}
		// if non writable prop
		if (!canWrite) {
			if (existsCustomEditor) {
				return getInput(getDisabledWriterComponent(), true);// read-only
			} else {
				return getDisabledWriterComponent();
			}
		}
		final boolean editable = (editor instanceof EnhancedPropertyEditor)
				&& ((EnhancedPropertyEditor) editor)
						.supportsEditingTaggedValues();
		String[] tags; // Tags
		if ((tags = editor.getTags()) != null) {
			return getInput(getInputTag(tags, stringValue, editable),
					existsCustomEditor);
		}
		if (canEditAsText) {
			return getInput(getInputLine(stringValue == null ? "???"
					: stringValue, true), // NOI18N
					existsCustomEditor);
		}
		if (existsCustomEditor) {
			return getInput(getDisabledWriterComponent(), true);
		}
		return getDisabledWriterComponent();
	}

	/**
	 * Getter for property changeImmediate. IF this is true the changes made in
	 * the property editor are immediately propagated to the value of the
	 * property (to the property model).
	 * 
	 * @return Value of property changeImmediate.
	 */
	public boolean isChangeImmediate() {
		return changeImmediate;
	}

	/** Getter for <code>isWriteState</code> property. */
	boolean isWriteState() {
		return isWriteState;
	}

	private void notifyChangeListeners() {
		if (changeListeners == null) {
			return;
		}
		final ChangeEvent e = new ChangeEvent(this);
		for (final Iterator<?> it = changeListeners.iterator(); it.hasNext();) {
			((ChangeListener) it.next()).stateChanged(e);
		}
	}

	/**
	 * Processes <code>Throwable</code> thrown from <code>setAsText</code>
	 * or <code>setValue</code> call on <code>editor</code>. Helper method.
	 */
	private void processThrowable(Throwable throwable) {
		if (throwable instanceof ThreadDeath) {
			throw (ThreadDeath) throwable;
		}
	}

	/**
	 * Refreshes the view after the property has changed.
	 */
	void refresh() {
		if (isReadState) {
			// [PENDING] anything more reasonable than reset here?
		}
		if (isWriteState
				&& ((preferences & PropertyPanel.PREF_INPUT_STATE) == 0)) {
			// When leaving 'write state' setReadState will be called directly,
			// see listeners at the bottom.
			return;
		}
		reset();
	}

	/* Overriden to forward focus request to focus traversable subcomponent. */
	@Override
	public void requestFocus() {
		final ContainerOrderFocusTraversalPolicy policy = new ContainerOrderFocusTraversalPolicy();
		policy.getDefaultComponent(this).requestFocus();
	}

	/**
	 * Set the panel to the read state with fresh read component.
	 */
	private void reset() {
		isWriteState = false;
		isReadState = false;
		if ((preferences & PropertyPanel.PREF_INPUT_STATE) != 0) {
			setWriteState();
			return;
		}
		if (((preferences & PropertyPanel.PREF_CUSTOM_EDITOR) != 0)
				&& (editor != null)) {
			final Component c = editor.getCustomEditor();
			if ((c != null) && (c.getParent() != this)) {
				removeAll();
				add(c, BorderLayout.CENTER);
				validate();
				return;
			}
		}
		setReadState();
	}

	/**
	 * Sets whether or not all components within selected component will be
	 * enabled.
	 * 
	 * @param cmp
	 *            component to be enabled/disabled
	 * @param enabled
	 *            flag defining the action.
	 */
	private void setComponentEnabled(Component cmp, boolean enabled) {
		cmp.setEnabled(enabled);
		if (Container.class.isAssignableFrom(cmp.getClass())) {
			final Container cont = (Container) cmp;
			final Component[] comp = cont.getComponents();
			for (Component element : comp) {
				element.setEnabled(enabled);
				setComponentEnabled(element, enabled);
			}
		}
	}

	/**
	 * Set the foreground color of disabled properties.
	 * 
	 * @param color
	 *            the new color
	 */
	void setDisabledColor(Color color) {
		disabledColor = color;
		reset();
	}

	// bugfix# 10171 added setEnabled() and setComponentEnabled() methods
	/**
	 * Sets whether or not this component is enabled.
	 * 
	 * all panel components gets disabled when enabled parameter is set false
	 * 
	 * @param enabled
	 *            flag defining the action.
	 */
	@Override
	public void setEnabled(boolean enabled) {
		final Component[] comp = getComponents();
		if (comp != null) {
			for (Component element : comp) {
				setComponentEnabled(element, enabled);
			}
		}
		super.setEnabled(enabled);
	}

	/**
	 * Set the foreground color of values.
	 * 
	 * @param color
	 *            the new color
	 */
	void setForegroundColor(Color color) {
		this.foregroundColor = color;
		reset();
	}

	/**
	 * Setter for property model.
	 * 
	 * @param model
	 *            New value of property model.
	 */
	public void setModel(PropertyModel model) {
		final PropertyModel oldModel = this.model;
		this.model = model;
		oldModel.removePropertyChangeListener(getModelListener());
		model.addPropertyChangeListener(getModelListener());
		updateEditor();
		reset();
		firePropertyChange(PropertyPanel.PROP_MODEL, oldModel, model);
	}

	/** Sets painting style. */
	void setPaintingStyle(int style) {
		paintingStyle = style;
		reset();
	}

	/**
	 * Set whether buttons in sheet should be plastic.
	 * 
	 * @param plastic
	 *            true if so
	 */
	void setPlastic(boolean plastic) {
		this.plastic = plastic;
		reset();
	}

	/**
	 * Setter for visual preferences in displaying of the value of the property.
	 * 
	 * @param pref
	 *            PREF_XXXX constants
	 */
	public void setPreferences(int preferences) {
		this.preferences = preferences;
		readComponent = null;
		reset();
		/*
		 * firePropertyChange( PROP_PREFERENCES, new Integer (oldPreferences),
		 * new Integer (preferences) );
		 */
	}

	/**
	 * Switches from writing component to reading one.
	 */
	void setReadState() {
		if (isReadState) {
			return;
		}
		if ((preferences & PropertyPanel.PREF_INPUT_STATE) != 0) {
			return;
		}
		if ((preferences & PropertyPanel.PREF_CUSTOM_EDITOR) != 0) {
			return;
		}
		isWriteState = false;
		isReadState = false;
		removeAll();
		readComponent = getReaderComponent();
		readComponent.addSheetButtonListener(getReadComponentListener());
		// bugfix# 10171 setComponentEnabled() call before add()...
		setComponentEnabled(readComponent, isEnabled());
		readComponent.setToolTipText(getPanelToolTipText());
		add(readComponent, BorderLayout.CENTER);
		revalidate();
		repaint();
		// Bug fix #13933 needles requesting of focus for cases when only
		// refresh
		// needed. In case of problems some better solution is needed.
		// Sure is only the thing, focus may not be requested when coming from
		// refresh().
		// requestFocus();
		isReadState = true;
		isWriteState = false;
	}

	/**
	 * Switches from reading component to writing one.
	 */
	void setWriteState() {
		if (isWriteState) {
			return;
		}
		if ((preferences & PropertyPanel.PREF_READ_ONLY) != 0) {
			return;
		}
		if ((preferences & PropertyPanel.PREF_CUSTOM_EDITOR) != 0) {
			return;
		}
		isReadState = false;
		isWriteState = false;
		removeAll();
		final JComponent c = getWriterComponent();
		add(c, BorderLayout.CENTER);
		// updateNeighbourPanels();
		revalidate();
		repaint();
		final Component focused = PropertyPanel.getDefaultFocusComponent(c);
		if (focused != null) {
			final WriteComponentListener l = getWriteComponentListener();
			focused.requestFocus();
			focused.removeFocusListener(l);
			focused.removeKeyListener(l);
			focused.addFocusListener(l);
			focused.addKeyListener(l);
		}
		isReadState = false;
		isWriteState = true;
	}

	/** Attempts to advance to next item in comboBox. */
	void tryToSelectNextTag() {
		if (comboBox == null) {
			return;
		}
		setWriteState();
		int index = comboBox.getSelectedIndex();
		index++;
		if (index >= comboBox.getItemCount()) {
			index = 0;
		}
		comboBox.setSelectedIndex(index);
	}

	// private methods -------------------------------------------------------
	/** Updates all neighbour panels. */
	/**
	 * Update the current property editor depending on the model.
	 */
	@SuppressWarnings("unchecked")
	private void updateEditor() {
		// final PropertyEditor oldEditor = editor;
		if (editor != null) {
			editor.removePropertyChangeListener(getEditorListener());
		}
		// find new editor
		editor = null;
		if (editor == null) {
			final Class editorClass = model.getPropertyEditorClass();
			if (editorClass != null) {
				try {
					editor = (PropertyEditor) editorClass.newInstance();
				} catch (final Exception e) {
					// PropertyDialogManager.notify(e);
				}
			}
		}
		/*
		 * if (editor == null) { Class propertyTypeClass =
		 * model.getPropertyType();
		 * 
		 * if (propertyTypeClass != null) { editor =
		 * PropertyEditorManager.findEditor(propertyTypeClass); } }
		 */
		if (editor != null) {
			try {
				if (editor instanceof ChoicePropertyEditor) {
					((ChoicePropertyEditor) editor).setTags(model.getTags());
				}
				if (canRead) {
					editor.setValue(model.getValue());
				}
			} /*
				 * catch (ProxyNode.DifferentValuesException dve) {
				 * differentValues = true; }
				 */
			catch (final Exception e) {
				processThrowable(e);
			}
			if (canWrite) {
				editor.addPropertyChangeListener(getEditorListener());
			}
		}
		// fire the change
		// firePropertyChange(PROP_PROPERTY_EDITOR, oldEditor, editor);
	}

	/**
	 * Updates the value if
	 * <code>org.openide.explorer.propertysheet.editors.EnhancedCustomPropertyEditor</code>
	 * is used.
	 */
	public void updateValue() {
		if (editor == null) {
			return;
		}
		if (editor.supportsCustomEditor()) {
			final Component customEditor = editor.getCustomEditor();
			if (customEditor instanceof EnhancedCustomPropertyEditor) {
				try {
					final Object value = ((EnhancedCustomPropertyEditor) customEditor)
							.getPropertyValue();
					editor.setValue(value);
					notifyChangeListeners();
				} catch (final IllegalStateException ise) {
					// PropertyDialogManager.notify(ise);
				}
			}
		}
	}
}
