package yaes.ui.propertysheet;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.PropertyChangeListener;
import java.util.Map;
import java.util.Set;
import java.util.Vector;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JToolBar;
import javax.swing.event.ChangeListener;

public class PropertySheet extends JPanel {
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	protected JToolBar toolBar = new JToolBar();
	protected TableSheet tableSheet;
	private final ImageIcon sortByNamesIcon = new ImageIcon(
			"com/cplane/ic/gui/resources/smallimages/sortedByNames.gif");
	private final ImageIcon sortByTypesIcon = new ImageIcon(
			"com/cplane/ic/gui/resources/smallimages/sortedByTypes.gif");
	protected JButton btnSortByName = new JButton();
	protected JButton btnSortByType = new JButton();
	@SuppressWarnings("unchecked")
	protected java.util.List order = null;
	@SuppressWarnings("unchecked")
	protected Set readOnly = null;
	@SuppressWarnings("unchecked")
	protected Vector<ChangeListener> changeListeners = new Vector<ChangeListener>();

	public PropertySheet() {
	}

	@SuppressWarnings("unchecked")
	public PropertySheet(Map map) {
		tableSheet = new TableSheet(map);
		tableSheet.setChangeListeners(changeListeners);
		init();
	}

	@SuppressWarnings("unchecked")
	public PropertySheet(Map map, java.util.List order) {
		tableSheet = new TableSheet(map, order);
		tableSheet.setChangeListeners(changeListeners);
		this.order = order;
		toolBar.setVisible(false);
		init();
	}

	@SuppressWarnings("unchecked")
	public PropertySheet(Map map, java.util.List order, Set readOnly) {
		tableSheet = new TableSheet(map, order, readOnly);
		tableSheet.setChangeListeners(changeListeners);
		this.order = order;
		this.readOnly = readOnly;
		toolBar.setVisible(false);
		init();
	}

	public void addChangeListener(ChangeListener listener) {
		changeListeners.add(listener);
		tableSheet.setChangeListeners(changeListeners);
	}

	@Override
	public void addPropertyChangeListener(PropertyChangeListener listener) {
		tableSheet.addPropertyChangeListener(listener);
	}

	protected void createPropertySheet(ExPropertyTableModel model) {
		tableSheet = new TableSheet(model);
		tableSheet.setChangeListeners(changeListeners);
		init();
	}

	private void init() {
		initButtons();
		initToolBar();
		final GridBagLayout gl = new GridBagLayout();
		this.setLayout(gl);
		final GridBagConstraints gbc = new GridBagConstraints();
		gbc.weighty = 0;
		gbc.weightx = 1;
		gbc.fill = GridBagConstraints.HORIZONTAL;
		gbc.gridx = 0;
		gbc.gridy = 0;
		gl.setConstraints(toolBar, gbc);
		gbc.weighty = 1;
		gbc.weightx = 1;
		gbc.fill = GridBagConstraints.BOTH;
		gbc.gridx = 0;
		gbc.gridy = 1;
		gl.setConstraints(tableSheet, gbc);
		this.add(toolBar);
		this.add(tableSheet);
	}

	protected void initButtons() {
		btnSortByType.setIcon(sortByTypesIcon);
		btnSortByType.setToolTipText("Sort by Type");
		btnSortByName.setIcon(sortByNamesIcon);
		btnSortByName.setToolTipText("Sort by Name");
		btnSortByName.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				tableSheet.sortByName();
			}
		});
		btnSortByType.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				tableSheet.sortByType();
			}
		});
	}

	protected void initToolBar() {
		toolBar.add(btnSortByName);
		toolBar.addSeparator();
		toolBar.add(btnSortByType);
	}

	// to be used by ResourceAttributeSheet
	@SuppressWarnings("unchecked")
	public void setMap(Map map) {
		tableSheet = new TableSheet(map);
		tableSheet.setChangeListeners(changeListeners);
		init();
	}

	@SuppressWarnings("unchecked")
	public void setOrder(java.util.List order) {
		this.order = order;
		toolBar.setVisible(false);
		tableSheet.setOrder(order);
	}

	@SuppressWarnings("unchecked")
	public void setReadOnly(Set readOnly) {
		this.readOnly = readOnly;
		tableSheet.setReadOnly(readOnly);
	}
}
