package yaes.ui.propertysheet.editors;

/**
 * Title:
 * Description:
 * Copyright:    Copyright (c) 2001
 * Company:
 * @author
 * @version 1.0
 */
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.beans.PropertyEditor;
import java.io.File;

import javax.swing.JFileChooser;

public class FileEditor implements PropertyEditor {
	class ExFileChooser extends JFileChooser implements CustomEditorPresenter {
		/**
		 * 
		 */
		private static final long serialVersionUID = 1L;

		public void showDialog(Component parent) {
			super.showDialog(parent, "Select File");
		}
	}

	private File file;
	private final PropertyChangeSupport support;

	private ExFileChooser chooser;

	public FileEditor() {
		support = new PropertyChangeSupport(this);
	}

	public void addPropertyChangeListener(
			PropertyChangeListener propertychangelistener) {
		support.addPropertyChangeListener(propertychangelistener);
	}

	/* gets string value */
	public String getAsText() {
		final Object obj = getValue();
		if (obj instanceof File) {
			return obj.toString();
		}
		return null;
	}

	public Component getCustomEditor() {
		chooser = new ExFileChooser();
		// chooser.setApproveButtonText (getString ("Select File"));
		// chooser.setApproveButtonToolTipText (getString
		// ("CTL_ApproveSelectToolTip"));
		initChooser();
		chooser.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
				if (JFileChooser.APPROVE_SELECTION.equals(evt
						.getActionCommand())) {
					final File f = chooser.getSelectedFile();
					setValue(f);
				}
			}
		});
		return chooser;
	}

	public String getJavaInitializationString() {
		final File value = (File) getValue();
		if (value == null) {
			return "null"; // NOI18N
		} else {
			// [PENDING] not a full escape of filenames, but enough to at least
			// handle normal Windows backslashes
			return "new java.io.File (\"" + getAsText() + ")";
			// this takes care of Windows file path,should fix later
			// Utilities.replaceString (value.getAbsolutePath (), "\\", "\\\\")
			// + "\")";
		}
	}

	public String[] getTags() {
		return null;
	}

	public Object getValue() {
		return file;
	}

	private void initChooser() {
		final File originalFile = (File) getValue();
		if (originalFile != null) {
			if (originalFile.isDirectory()) {
				chooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
			} else {
				chooser
						.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);
			}
			if ((originalFile != null) && (originalFile.getParent() != null)) {
				chooser.setCurrentDirectory(new File(originalFile.getParent()));
			}
			chooser.setSelectedFile(originalFile);
		}
	}

	public boolean isPaintable() {
		return false;
	}

	public void paintValue(Graphics g, Rectangle rectangle) {
	}

	public Dimension preferredSize() {
		return chooser.getPreferredSize();
	}

	public void removePropertyChangeListener(
			PropertyChangeListener propertychangelistener) {
		support.removePropertyChangeListener(propertychangelistener);
	}

	/* sets new value */
	public void setAsText(String s) {
		setValue(new File(s));
	}

	public void setValue(Object obj) {
		file = (File) obj;
		support.firePropertyChange("", null, null);
	}

	public boolean supportsCustomEditor() {
		// if property read only,has to return false
		return true;
	}
}
