package yaes.ui.propertysheet.editors;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Rectangle;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.beans.PropertyEditor;

import javax.swing.JPanel;

import yaes.ui.propertysheet.ErrorPresenter;

public class FontEditor extends JPanel implements PropertyEditor,
		CustomEditorWidget {
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	private final JFontChooser chooser;
	private Font font;

	private final String sampleText;

	private final PropertyChangeSupport support;;

	public FontEditor() {
		sampleText = "Abcde...";
		support = new PropertyChangeSupport(this);
		chooser = new JFontChooser();
		chooser.addPropertyChangeListener(new PropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent e) {
				font = chooser.getFontValue();
			}
		});
		final Dimension dim = chooser.getPreferredSize();
		final int width = dim.width;
		final int height = dim.height;
		final int vpad = 20;
		this.setPreferredSize(new Dimension(width, height + vpad));
		add(chooser);
	}

	public void actionOk() {
	}

	@Override
	public void addPropertyChangeListener(
			PropertyChangeListener propertychangelistener) {
		support.addPropertyChangeListener(propertychangelistener);
	}

	public String getAsText() {
		return null;
	}

	public Component getCustomEditor() {
		return this;
	}

	public String getJavaInitializationString() {
		return "new java.awt.Font(\"" + font.getFamily() + "\", "
				+ font.getStyle() + ", " + font.getSize() + ")";
	}

	public String[] getTags() {
		return null;
	}

	public Object getValue() {
		return chooser.getFontValue();
	}

	public String getWidgetTitle() {
		return "Select Font";
	}

	public boolean isPaintable() {
		return true;
	}

	public void paintValue(Graphics g, Rectangle rectangle) {
		final Font font1 = g.getFont();
		g.setFont(font);
		final FontMetrics fontmetrics = g.getFontMetrics();
		final int i = (rectangle.height - fontmetrics.getAscent()) / 2;
		g.drawString(sampleText, 1, rectangle.height - i);
		g.setFont(font1);
	}

	@Override
	public void removePropertyChangeListener(
			PropertyChangeListener propertychangelistener) {
		support.removePropertyChangeListener(propertychangelistener);
	}

	public void setAsText(String s) throws IllegalArgumentException {
		throw new IllegalArgumentException(s);
	}

	public void setErrorPresenter(ErrorPresenter errorPresenter) {
	}

	public void setValue(Object obj) {
		font = (Font) obj;
		chooser.setFontValue(font);
		support.firePropertyChange("", null, null);
	}

	public boolean supportsCustomEditor() {
		return true;
	}
}
