package yaes.ui.propertysheet.editors;

import java.awt.Component;
import java.awt.Dialog;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Frame;
import java.awt.GraphicsEnvironment;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.font.TextAttribute;
import java.util.HashMap;
import java.util.Map;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JTextArea;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;
import javax.swing.WindowConstants;

//import gate.util.*;
public class JFontChooser extends JPanel {
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	/**
	 * 198 * Test code 199
	 */
	public static void main(String args[]) {
		try {
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		} catch (final Exception e) {
			e.printStackTrace();
		}
		final JFrame frame = new JFrame("Foo frame");
		frame.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
		final JButton btn = new JButton("Show dialog");
		btn.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				System.out.println(JFontChooser.showDialog(frame, "Fonter",
						UIManager.getFont("Button.font")));
			}
		});
		frame.getContentPane().add(btn);
		frame.setSize(new Dimension(300, 300));
		frame.setVisible(true);
		System.out.println("Font: " + UIManager.getFont("Button.font"));
		JFontChooser.showDialog(frame, "Fonter", UIManager
				.getFont("Button.font"));
	}// main

	public static Font showDialog(Component parent, String title,
			Font initialfont) {
		Window windowParent;
		if (parent instanceof Window) {
			windowParent = (Window) parent;
		} else {
			windowParent = SwingUtilities.getWindowAncestor(parent);
		}
		if (windowParent == null) {
			throw new IllegalArgumentException(
					"The supplied parent component has no window ancestor");
		}
		final JDialog dialog;
		if (windowParent instanceof Frame) {
			dialog = new JDialog((Frame) windowParent, title, true);
		} else {
			dialog = new JDialog((Dialog) windowParent, title, true);
		}
		dialog.getContentPane().setLayout(
				new BoxLayout(dialog.getContentPane(), BoxLayout.Y_AXIS));
		final JFontChooser fontChooser = new JFontChooser(initialfont);
		dialog.getContentPane().add(fontChooser);
		final JButton okBtn = new JButton("OK");
		final JButton cancelBtn = new JButton("Cancel");
		final JPanel buttonsBox = new JPanel();
		buttonsBox.setLayout(new BoxLayout(buttonsBox, BoxLayout.X_AXIS));
		buttonsBox.add(Box.createHorizontalGlue());
		buttonsBox.add(okBtn);
		buttonsBox.add(Box.createHorizontalStrut(30));
		buttonsBox.add(cancelBtn);
		buttonsBox.add(Box.createHorizontalGlue());
		dialog.getContentPane().add(buttonsBox);
		dialog.pack();
		fontChooser.addComponentListener(new ComponentAdapter() {
			@Override
			public void componentResized(ComponentEvent e) {
				dialog.pack();
			}
		});
		okBtn.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				dialog.setVisible(false);
			}
		});
		cancelBtn.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				dialog.setVisible(false);
				fontChooser.setFontValue(null);
			}
		});
		dialog.setVisible(true);
		return fontChooser.getFontValue();
	}// showDialog

	JComboBox familyCombo;

	JCheckBox italicChk;

	JCheckBox boldChk;

	JComboBox sizeCombo;

	JTextArea sampleTextArea;

	private java.awt.Font fontValue;

	public JFontChooser() {
		this(UIManager.getFont("Button.font"));
	}

	public JFontChooser(Font initialFont) {
		initLocalData();
		initGuiComponents();
		initListeners();
		setFontValue(initialFont);
	}// public JFontChooser(Font initialFont)

	public java.awt.Font getFontValue() {
		return fontValue;
	}

	protected void initGuiComponents() {
		this.setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
		familyCombo = new JComboBox(GraphicsEnvironment
				.getLocalGraphicsEnvironment().getAvailableFontFamilyNames());
		familyCombo
				.setSelectedItem(UIManager.getFont("Label.font").getFamily());
		sizeCombo = new JComboBox(new String[] { "6", "8", "10", "12", "14",
				"16", "18", "20", "22", "24", "26" });
		sizeCombo.setSelectedItem(new Integer(UIManager.getFont("Label.font")
				.getSize()).toString());
		italicChk = new JCheckBox("<html><i>Italic</i></html>", false);
		boldChk = new JCheckBox("<html><i=b>Bold</b></html>", false);
		final JPanel fontBox = new JPanel();
		fontBox.setLayout(new BoxLayout(fontBox, BoxLayout.X_AXIS));
		fontBox.add(familyCombo);
		fontBox.add(sizeCombo);
		fontBox.setBorder(BorderFactory.createTitledBorder(" Font "));
		add(fontBox);
		add(Box.createVerticalStrut(10));
		final JPanel effectsBox = new JPanel();
		effectsBox.setLayout(new BoxLayout(effectsBox, BoxLayout.X_AXIS));
		effectsBox.add(italicChk);
		effectsBox.add(boldChk);
		effectsBox.setBorder(BorderFactory.createTitledBorder(" Effects "));
		add(effectsBox);
		add(Box.createVerticalStrut(10));
		sampleTextArea = new JTextArea("Type your sample here...");
		final JPanel samplePanel = new JPanel();
		samplePanel.setLayout(new BoxLayout(samplePanel, BoxLayout.X_AXIS));
		// samplePanel.add(new JScrollPane(sampleTextArea));
		samplePanel.add(sampleTextArea);
		samplePanel.setBorder(BorderFactory.createTitledBorder(" Sample "));
		add(samplePanel);
		add(Box.createVerticalStrut(10));
	}// initGuiComponents()

	protected void initListeners() {
		familyCombo.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				updateFont();
			}
		});
		sizeCombo.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				updateFont();
			}
		});
		boldChk.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				updateFont();
			}
		});
		italicChk.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				updateFont();
			}
		});
	}// initListeners()

	protected void initLocalData() {
	}

	public void setFontValue(java.awt.Font newfontValue) {
		boldChk.setSelected(newfontValue.isBold());
		italicChk.setSelected(newfontValue.isItalic());
		familyCombo.setSelectedItem(newfontValue.getName());
		sizeCombo.setSelectedItem(Integer.toString(newfontValue.getSize()));
		this.fontValue = newfontValue;
	}

	protected void updateFont() {
		final Map<TextAttribute, Object> fontAttrs = new HashMap<TextAttribute, Object>();
		fontAttrs.put(TextAttribute.FAMILY, familyCombo.getSelectedItem());
		fontAttrs.put(TextAttribute.SIZE, new Float((String) sizeCombo
				.getSelectedItem()));
		if (boldChk.isSelected()) {
			fontAttrs.put(TextAttribute.WEIGHT, TextAttribute.WEIGHT_BOLD);
		} else {
			fontAttrs.put(TextAttribute.WEIGHT, TextAttribute.WEIGHT_REGULAR);
		}
		if (italicChk.isSelected()) {
			fontAttrs.put(TextAttribute.POSTURE, TextAttribute.POSTURE_OBLIQUE);
		} else {
			fontAttrs.put(TextAttribute.POSTURE, TextAttribute.POSTURE_REGULAR);
		}
		final Font newFont = new Font(fontAttrs);
		final Font oldFont = fontValue;
		fontValue = newFont;
		sampleTextArea.setFont(newFont);
		final String text = sampleTextArea.getText();
		sampleTextArea.setText("");
		sampleTextArea.setText(text);
		sampleTextArea.repaint(100);
		firePropertyChange("fontValue", oldFont, newFont);
	}// updateFont()
}// class JFontChooser extends JPanel
