package yaes.ui.visualization;

import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.Rectangle;
import java.awt.image.RenderedImage;
import java.io.File;
import java.io.IOException;

import javax.imageio.ImageIO;
import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JSplitPane;

/**
 * @author Lotzi Boloni
 * 
 * This a class which is used to visualize the scenarios
 * 
 * 
 */
public class VisualMap {

	private JFrame frame = null;
	private VisualMapPanel panel = null;
	private VisualMapInspector inspector = null;
	// helper
	IInspector genericInspector = new GenericInspector();

	public VisualMap(int width, int height, String icon, String title) {
		frame = new JFrame();
		final JSplitPane splitPane = new JSplitPane();
		frame.getContentPane().add(splitPane);
		panel = new VisualMapPanel(this);
		splitPane.setLeftComponent(panel);
		inspector = new VisualMapInspector(this);
		splitPane.setRightComponent(inspector);
		splitPane.setDividerLocation(0.8);
		JFrame.setDefaultLookAndFeelDecorated(true);
		if (icon != null) {
			frame.setIconImage(new ImageIcon(icon).getImage());
		}
		frame.pack();
		frame.setResizable(false);
		frame.setVisible(true);
		frame.setTitle(title);
	}

	/**
	 * Simplified addObject: adds objects based on their types.
	 * 
	 * @param o
	 */
	public synchronized void addObject(Object o) {
		panel.addObject(o);
		inspector.addObject(o, genericInspector);
	}

	/**
	 * Adds an object with the associated painter.
	 * 
	 * @param o
	 * @param painter
	 */
	public synchronized void addObject(Object o, IPainter painter) {
		panel.addObject(o, painter);
		inspector.addObject(o, genericInspector);
	}

	public JFrame getFrame() {
		return this.frame;
	}

	/**
	 * Removes an object from the visualization
	 * 
	 * @param o
	 */
	public synchronized void removeObject(Object o) {
		panel.removeObject(o);
		inspector.removeObject(o);
	}

	public void repaint() {
		panel.invalidate();
		panel.repaint();
	}

	/**
	 * Saves an image
	 * 
	 * @param fileName
	 * @throws IOException
	 */
	public void saveImage(String fileName) throws IOException {
		final Rectangle rect = frame.getBounds();
		final Image fileImage = frame.createImage(rect.width, rect.height);
		final Graphics g = fileImage.getGraphics();
		// write to the image
		panel.paintVisualMap((Graphics2D) g);
		// write it out in the format you want
		ImageIO.write((RenderedImage) fileImage, "png", new File(fileName));
		// dispose of the graphics content
		g.dispose();
	}

	public void setDimension(int width, int height) {
		frame.setBounds(0, 0, width, height);
		panel.setBounds(0, 0, width, height);
	}

	/**
	 * @param b
	 */
	public void setVisible(boolean b) {
		frame.setVisible(b);
	}
}