package yaes.ui.visualization.application.sensornetwork;


import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.geom.Point2D;
import java.util.List;

import yaes.framework.agent.ACLMessage;
import yaes.framework.world.sensornetwork.ActuatorNode;
import yaes.framework.world.sensornetwork.Perception;
import yaes.framework.world.sensornetwork.SensorNetworkWorld;
import yaes.framework.world.sensornetwork.SensorNode;
import yaes.ui.visualization.MovingObjectPainter;
import yaes.ui.visualization.VisualMapPanel;

/**
 * Implements a painter for a mobile node in the sensor network
 * 
 * @author Lotzi Boloni <lboloni@eecs.ucf.edu>
 *
 */
public class MobileNodePainter extends MovingObjectPainter {
	private final SensorNetworkWorld sensorWorld;
	private final PainterConfiguration painterConf;

	public MobileNodePainter(SensorNetworkWorld sensingManager,
			PainterConfiguration painterConf) {
		this.sensorWorld = sensingManager;
		this.painterConf = painterConf;
	}

	@Override
	public void paint(Graphics2D g, Object o, VisualMapPanel panel) {
		super.paint(g, o, panel);
		if (o instanceof ActuatorNode) {
			final ActuatorNode node = (ActuatorNode) o;
			final String name = node.getName();
			g.setColor(Color.BLACK);
			
			Point2D.Double point = node.getLocation().asPoint();
			panel.getTheTransform().transform(point, point);
			
			g.drawString(name, (int)point.getX(), (int)point.getY());
			paintPerceptions(g, node, panel);
		}
	}

	/**
	 * Paints the perceptions
	 * 
	 * @param g
	 * @param agent
	 */
	private void paintPerceptions(Graphics2D g, ActuatorNode node, VisualMapPanel panel) {
		final Point2D.Double point = node.getLocation().getRoundedLocation().asPoint();
		panel.getTheTransform().transform(point, point);
		final int x = (int) point.getX();
		final int y = (int) point.getY();
		final List<Perception> perceptions =
				sensorWorld.getSensingHistory(node).inspectLastPerceptions(
						(Integer)painterConf.get(PainterConfiguration.PCONF_SECONDS_TO_CONSIDER));
		for (final Perception p : perceptions) {
			switch (p.getType()) {
			case Overhearing: {
				// blink the sensor range yellow
				g.setColor((Color)painterConf.get(PainterConfiguration.PCONF_OVERHEARING_COLOR));
				final int radius = 10;
				g.fillOval((x - radius), (y - radius), (2 * radius),
						(2 * radius));
				break;
			}
			case ReceivedMessage: {
				// draw a line to the node which sent the message
				final ACLMessage m = p.getMessage();
				final SensorNode sender =
						(SensorNode) m
								.getValue(SensorNetworkWorld.HIDDEN_FIELD_SENDER_NODE);
				final Point2D.Double senderLocation =
						sender.getLocation().getRoundedLocation().asPoint();
				panel.getTheTransform().transform(senderLocation, senderLocation);								
				g.drawLine(x, y, (int) senderLocation.getX(),
						(int) senderLocation.getY());
				break;
			}
			}
		}
	}
}
